from handyfuncs.automation import (
    write_to_log
    , exec_fn
    , create_empty_log_file
    , run_subprocess
    , create_empty_text_file
    , append_to_text_file
    , write_to_new_text_file
    , current_method_name
    )

from handyfuncs.manipulation import (
    number_of_words_in_string
    , all_same_character
    , get_initials
    , literal_return
    , explode_frame_list_column
    , convert_dict_to_list
    , merge_dictionaries
    )